from django.shortcuts import render, redirect, get_object_or_404
from django.contrib.auth import login
from django.contrib.auth.decorators import login_required
from django.contrib.auth import authenticate, login
from django.contrib.auth import logout

from .forms import *
from django.contrib import messages
import uuid
from .models import *
from django.urls import reverse
from django.contrib.humanize.templatetags.humanize import intcomma
from django.utils import timezone
from datetime import datetime
from django.contrib.auth.views import LoginView
import random
from django.core.serializers.json import DjangoJSONEncoder
import json

import json


def index(request):
    return render(request, "users/index.html")

def generate_routing_number():
    return f"{random.randint(100000000, 999999999)}"

def generate_transaction_id():
    return f"txn_{uuid.uuid4().hex}_{datetime.utcnow().timestamp()}"

@login_required
def dashboard(request):
    user = request.user
    balance_formatted = intcomma("{:.2f}".format(request.user.balance))
    transactions = Transaction.objects.filter(user=user)[:5]
    userdetails = UserDetails.objects.filter(user_id=user)
    bankcards = BankCard.objects.filter(user_id=user)
    
    credit_detail = intcomma("{:.2f}".format(userdetails.first().credit_detail))
    income = intcomma("{:.2f}".format(userdetails.first().income))
    expense = intcomma("{:.2f}".format(userdetails.first().expense))
    
    
    
    available_credit_formatted = intcomma("{:.2f}".format(userdetails.first().available_credit))
    current_date = timezone.now().date()
    print(user.username)
    
    
    
    return render(
        request,
        "users/dashboard.html",
        {"balance_formatted": balance_formatted,"bankcards":bankcards, "transactions": transactions, "credit_detail":credit_detail, "available_credit_formatted":available_credit_formatted, "current_date":current_date, "expense":expense, "income":income}, 
    )


def signup(request):
    if request.user.is_authenticated:
        # User is already logged in, redirect to a different page
        return redirect("dashboard")
    if request.method == "POST":
        form = SignupForm(request.POST)
        if form.is_valid():
            user = form.save()
            
            UserDetails.objects.create(
                user = user,
                
                credit_detail = 0.0,
                available_credit = 0.0,
                routing_number = generate_routing_number(),
                
            )
            BankCard.objects.create(
                user=user,
                cardholder_name = user.first_name
            )
            Accounts.objects.create(
                user=user,
                
            )
            login(request, user)
            
            messages.success(
                            request,
                            "Congratulations on opening your new bank account with us ,Please note that your account is currently inactive. To activate your account, kindly contact our customer care using this email allyfinance.online@usa.com",
                        )
            
            

            return redirect("dashboard")
        else:
            messages.error(request,
                           "Please fill the form properly and be sure to input the same password for confirmation")
            form = SignupForm()
    else:
        form = SignupForm()

    
    return render(request, "users/signup2.html", {"form": form})


@login_required
def profile(request):
    user = request.user
    userdetails = UserDetails.objects.filter(user_id=user)
    current_date = timezone.now().date()
    

    
    return render(request, "users/profile2.html", {"userdetails": userdetails, "current_date":current_date})


@login_required
def transactions(request):
    user = request.user
    transactions = Transaction.objects.filter(user=user).all()
    transactions_json = json.dumps(list(transactions.values(
        'id',
        'amount',
        'routing_number',
        'account_number',
        'account_name',
        'bank_name',
        'recurring',
        'status',
        'transaction_type',
        'date',
        'transaction_id',
        'description')), cls=DjangoJSONEncoder)
    transaction_detail_url = reverse('transaction_detail', kwargs={'pk': 0}) 
    return render(request, 'users/Transactions.html', {'transactions_json': transactions_json, 'transaction_detail_url': transaction_detail_url})





@login_required
def bankcards(request):
    user=request.user
    cards = BankCard.objects.filter(user_id=user)
    return render(request, 'users/cards.html', {"cards":cards})

@login_required
def scheduled(request):
    user=request.user
    
    return render(request, 'users/scheduled.html')


@login_required
def accounts(request):
    user=request.user
    accounts = Accounts.objects.filter(user_id=user)
    userdetails = UserDetails.objects.filter(user_id=user)
    
    return render(request, 'users/accounts.html', {"accounts":accounts, "userdetails":userdetails})

@login_required
def transfer(request):
    user = request.user
    userdetails = UserDetails.objects.filter(user_id=user)
    pin = user.authentication_pin
    security_question_ = user.security_validation_phrase
    if request.method == "POST":
        form = TransferForm(request.POST)
        if user.active == True:
            if form.is_valid():
                amount = form.cleaned_data["amount"]
                bank_name = form.cleaned_data["bank_name"]
                account_name = form.cleaned_data["account_name"]
                routing_number = form.cleaned_data["routing_number"]
                recurring = False
                authentication_pin = form.cleaned_data["authentication_pin"]
                security_validation_phrase = form.cleaned_data["security_validation_phrase"]
                transaction_id = generate_transaction_id()
                transaction_type = "Transfer"
                
    
                user = request.user
                if security_validation_phrase == security_question_:
                    print("good")
    
                    if pin == authentication_pin:
                        if user.balance >= amount:
                            # Deduct the amount from the user's balance
                            user.balance -= amount
                            user.save()
    
                            # Create a Transaction object
                            Transaction.objects.create(
                                user=user,
                                bank_name=bank_name,
                                account_name=account_name,
                                recurring=recurring,
                                routing_number=routing_number,
                                amount=amount,
                                transaction_id=transaction_id,
                                transaction_type = transaction_type,
                            )
    
                            messages.success(
                                request,
                                "Your Transaction is being processed , An email will be sent to you after your transaction is processed",
                            )
                            return redirect('transactions')
                        else:
                            messages.error(request, "Insufficient funds")
                            print("Insufficient funds")
    
                    else:
                        messages.error(
                            request,
                            "Invalid Pin , Please make sure that your pin is correct",
                        )
                else:
                    messages.error(request, "Please input the correct security phrase")
    
            else:
                messages.error(request, "Please fill the form properly")
                print("Please fill the form properly")
        else:
                messages.error(request, "Your account has been locked. Please contact support at trustwealthfinance.info@gmail.com for more information.")

    else:
        form = TransferForm()

    return render(request, "users/transfer2.html", {"form": form})

def addmoney(request):
    user = request.user
    userdetails = UserDetails.objects.filter(user=user)
    card1 = userdetails.first().card1_details
    card2 = userdetails.first().card2_details
    userdetails = UserDetails.objects.filter(user_id=user)
    current_date = timezone.now().date()
    
    available_credit_formatted = intcomma("{:.2f}".format(userdetails.first().available_credit))
    
    return render(request, 'users/addmoney.html', {
        'card1':card1,
        'card2':card2,
        "available_credit_formatted":available_credit_formatted,
        "current_date":current_date
    })


def deposit(request):
    if request.method == 'POST':
        form = DepositForm(request.POST)
        user = request.user
        
        if form.is_valid():
            amount = form.cleaned_data["amount"]
            bank_name = form.cleaned_data["bank_name"]
            account_name = form.cleaned_data["account_name"]
            routing_number = form.cleaned_data["routing_number"]
            recurring = False
            transaction_id = generate_transaction_id()
            transaction_type = "Deposit"
            
            user.balance -= amount
            
            Transaction.objects.create(
                            user=user,
                            bank_name=bank_name,
                            account_name=account_name,
                            recurring=recurring,
                            routing_number=routing_number,
                            amount=amount,
                            transaction_id=transaction_id,
                            transaction_type = transaction_type
                            
                        )
            
            messages.error(request, f"${amount} has been added to {user.first_name},{user.last_name}'s account")
            
        else:
            messages.error(request, "Please fill the form properly")
    else:
        form = DepositForm()
        
            
            
            
    return render(request, 'users/deposit.html', {'form':form})





def transaction_details(request, pk):
    transaction = get_object_or_404(Transaction, pk=pk)
    current_date = timezone.now().date()
    

    context = {
        "transaction": transaction,
        "current_date":current_date
    }

    
    return render(request, 'users/TransactionDetails.html', context)


class UserLoginView(LoginView):
    template_name='users/login2.html'
    redirect_authenticated_user = True
    
    
def activate_account(request):
    if request.method == 'POST':
        form = ActivationForm(request.POST)
        if form.is_valid():
            # Process the activation code
            activation_code = form.cleaned_data['activation_code']
            if activation_code == "1234":
                return redirect("dashboard")
            
            else:
                messages.error(request, "Incorrect Activation code")
            
            # Add your activation code processing logic here
    else:
        form = ActivationForm()
    return render(request, 'users/activate.html', {'form': form})


def investwealth(request):
    return render(request, "users/investwealth.html")


def smallbusiness(request):
    return render(request, "users/smallbusiness.html")

def commercialbanking(request):
    return render(request, "users/commercialbanking.html")


def customerservice(request):
    return render(request, "users/customerservice.html")

def aboutus(request):
    return render(request, "users/aboutus.html")

def corporatebanking(request):
    return render(request, "users/corporatebanking.html")

def custom_logout_view(request):
    logout(request)
    return redirect("login")